package onebit.rlecodec;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;

import junit.framework.TestCase;

public class RLEInputStreamTest extends TestCase {

  public void testEmptyStream() throws IOException {
    byte[] data = new byte[0];
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      int b = is.read();
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }

  public void testCorruptedStream1() throws IOException {
    byte[] data = new byte[] { 0 };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      is.read();
      fail("IOException expected");
    } catch (IOException exception) {
      assertEquals("Stream exhausted prematurelly", exception.getMessage());
    } finally {
      is.close();
    }
  }

  public void testCorruptedStream2() throws IOException {
    byte[] data = new byte[] { 0, 0, 0 };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      int b = is.read();
      assertEquals(0, b);
      is.read();
      fail("IOException expected");
    } catch (IOException exception) {
      assertEquals("Stream exhausted prematurelly", exception.getMessage());
    } finally {
      is.close();
    }
  }

  public void testCorruptedStream3() throws IOException {
    byte[] data = new byte[] { -1 };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      is.read();
      fail("IOException expected");
    } catch (IOException exception) {
      assertEquals("Stream exhausted prematurelly", exception.getMessage());
    } finally {
      is.close();
    }
  }

  public void testCorruptedStream4() throws IOException {
    byte[] data = new byte[] { -1, 0, -1 };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      int b = is.read();
      assertEquals(0, b);
      is.read();
      fail("IOException expected");
    } catch (IOException exception) {
      assertEquals("Stream exhausted prematurelly", exception.getMessage());
    } finally {
      is.close();
    }
  }

  public void testCorruptedStream5() throws IOException {
    byte[] data = new byte[] { 0, 0, -1 };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      int b = is.read();
      assertEquals(0, b);
      is.read();
      fail("IOException expected");
    } catch (IOException exception) {
      assertEquals("Stream exhausted prematurelly", exception.getMessage());
    } finally {
      is.close();
    }
  }

  public void testCorruptedStream6() throws IOException {
    byte[] data = new byte[] { -1, 0, 0 };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      int b = is.read();
      assertEquals(0, b);
      is.read();
      fail("IOException expected");
    } catch (IOException exception) {
      assertEquals("Stream exhausted prematurelly", exception.getMessage());
    } finally {
      is.close();
    }
  }

  public void testCorruptedStream7() throws IOException {
    byte[] data = new byte[] { 1, 0 };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      int b = is.read();
      assertEquals(0, b);
      is.read();
      fail("IOException expected");
    } catch (IOException exception) {
      assertEquals("Stream exhausted prematurelly", exception.getMessage());
    } finally {
      is.close();
    }
  }

  public void testCorruptedStream8() throws IOException {
    byte[] data = new byte[] { -1, 0, 1, 0 };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      int b = is.read();
      assertEquals(0, b);
      int b2 = is.read();
      assertEquals(0, b2);
      is.read();
      fail("IOException expected");
    } catch (IOException exception) {
      assertEquals("Stream exhausted prematurelly", exception.getMessage());
    } finally {
      is.close();
    }
  }

  public void testDifferentBlockStream1() throws IOException {
    byte[] data = new byte[] { 0, (byte)0xff };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      byte[] d = new byte[1];
      int c = is.read(d);
      assertEquals(1, c);
      byte[] exp = new byte[] { (byte)0xff };
      assertTrue(Arrays.equals(exp, d));
      int b = is.read();
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }

  public void testDifferentBlockStream2() throws IOException {
    byte[] data = new byte[] { 1, (byte)0xff, (byte)0xfe };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      byte[] d = new byte[2];
      int c = is.read(d);
      assertEquals(2, c);
      byte[] exp = new byte[] { (byte)0xff, (byte)0xfe };
      assertTrue(Arrays.equals(exp, d));
      int b = is.read();
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }

  public void testDifferentBlockStream3() throws IOException {
    byte[] data = new byte[129];
    data[0] = 127;
    for (int i = 0; i < 128; i++) {
      data[1+i] = (byte)i;
    }
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      byte[] d = new byte[128];
      int c = is.read(d);
      assertEquals(128, c);
      byte[] exp = new byte[128];
      for (int i = 0; i < exp.length; i++) {
        exp[i] = (byte)i;
      }
      assertTrue(Arrays.equals(exp, d));
      int b = is.read();
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }

  public void testDifferentDifferentBlocksStream1() throws IOException {
    byte[] data = new byte[131];
    data[0] = 127;
    for (int i = 0; i < 128; i++) {
      data[1+i] = (byte)i;
    }
    data[129] = 0;
    data[130] = (byte)128;
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      byte[] d = new byte[129];
      int c = is.read(d);
      assertEquals(129, c);
      byte[] exp = new byte[129];
      for (int i = 0; i < exp.length; i++) {
        exp[i] = (byte)i;
      }
      assertTrue(Arrays.equals(exp, d));
      int b = is.read();
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }

  public void testDifferentDifferentBlocksStream2() throws IOException {
    byte[] data = new byte[132];
    data[0] = 127;
    for (int i = 0; i < 128; i++) {
      data[1+i] = (byte)i;
    }
    data[129] = 1;
    for (int i = 0; i < 2; i++) {
      data[130+i] = (byte)i;
    }
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      byte[] d = new byte[130];
      int c = is.read(d);
      assertEquals(130, c);
      byte[] exp = new byte[130];
      for (int i = 0; i < 128; i++) {
        exp[i] = (byte)i;
      }
      for (int i = 0; i < 2; i++) {
        exp[128+i] = (byte)i;
      }
      assertTrue(Arrays.equals(exp, d));
      int b = is.read();
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }

  public void testEqualsEqualsBlocksStream1() throws IOException {
    byte[] data = new byte[] { -128, (byte)0xff, -1, (byte)0xff };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      byte[] d = new byte[129];
      int c = is.read(d);
      assertEquals(129, c);
      byte[] exp = new byte[129];
      for (int i = 0; i < exp.length; i++) {
        exp[i] = (byte)0xff;
      }
      assertTrue(Arrays.equals(exp, d));
      int b = is.read();
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }

  public void testEqualsEqualsBlocksStream2() throws IOException {
    byte[] data = new byte[] { -128, (byte)0xff, -2, (byte)0xff };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      byte[] d = new byte[130];
      int c = is.read(d);
      assertEquals(130, c);
      byte[] exp = new byte[130];
      for (int i = 0; i < exp.length; i++) {
        exp[i] = (byte)0xff;
      }
      assertTrue(Arrays.equals(exp, d));
      int b = is.read();
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }

  public void testDifferentEqualsBlocksStream1() throws IOException {
    byte[] data = new byte[131];
    data[0] = 127;
    for (int i = 0; i < 128; i++) {
      data[1+i] = (byte)i;
    }
    data[129] = -1;
    data[130] = (byte)128;
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      byte[] d = new byte[129];
      int c = is.read(d);
      assertEquals(129, c);
      byte[] exp = new byte[129];
      for (int i = 0; i < exp.length; i++) {
        exp[i] = (byte)i;
      }
      assertTrue(Arrays.equals(exp, d));
      int b = is.read();
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }

  public void testDifferentEqualsBlocksStream2() throws IOException {
    byte[] data = new byte[131];
    data[0] = 127;
    for (int i = 0; i < 128; i++) {
      data[1+i] = (byte)i;
    }
    data[129] = -2;
    data[130] = (byte)128;
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      byte[] d = new byte[130];
      int c = is.read(d);
      assertEquals(130, c);
      byte[] exp = new byte[130];
      for (int i = 0; i < 128; i++) {
        exp[i] = (byte)i;
      }
      exp[128] = (byte)128;
      exp[129] = (byte)128;
      assertTrue(Arrays.equals(exp, d));
      int b = is.read();
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }

  public void testEqualsDifferentBlocksStream1() throws IOException {
    byte[] data = new byte[] { -128, (byte)0xff, 0, 1 };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      byte[] d = new byte[129];
      int c = is.read(d);
      assertEquals(129, c);
      byte[] exp = new byte[129];
      for (int i = 0; i < 128; i++) {
        exp[i] = (byte)0xff;
      }
      exp[128] = 1;
      assertTrue(Arrays.equals(exp, d));
      int b = is.read();
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }

  public void testEqualsDifferentBlocksStream2() throws IOException {
    byte[] data = new byte[] { -128, (byte)0xff, 1, 0, 1 };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      byte[] d = new byte[130];
      int c = is.read(d);
      assertEquals(130, c);
      byte[] exp = new byte[130];
      for (int i = 0; i < 128; i++) {
        exp[i] = (byte)0xff;
      }
      exp[128] = 0;
      exp[129] = 1;
      assertTrue(Arrays.equals(exp, d));
      int b = is.read();
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }

  public void testReadBiggerBlockFromStream1() throws IOException {
    byte[] data = new byte[] { 0, 0 };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      byte[] d = new byte[128];
      int c = is.read(d);
      assertEquals(1, c);
      assertEquals(0, d[0]);
      int b = is.read();
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }

  public void testReadBiggerBlockFromStream2() throws IOException {
    byte[] data = new byte[] { 0, 0 };
    InputStream is = new RLEInputStream(new ByteArrayInputStream(data));
    try {
      byte[] d = new byte[128];
      int c = is.read(d);
      assertEquals(1, c);
      assertEquals(0, d[0]);
      int b = is.read(d);
      assertEquals(-1, b);
    } finally {
      is.close();
    }
  }
}
