package onebit.rlecodec;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Arrays;

import junit.framework.TestCase;

public class RLEOutputStreamTest extends TestCase {

  public void testEmptyStream1() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    assertEquals(0, a.length);
  }

  public void testEmptyStream2() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(new byte[0]);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    assertEquals(0, a.length);
  }

  public void testOneByteStream1() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 0, 0 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testOneByteStream2() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(1);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 0, 1 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testOneByteStream3() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0xff);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 0, (byte)0xff };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testTwoDifferentBytesStream1() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0);
    rleos.write(1);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 1, 0, 1 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testTwoDifferentBytesStream2() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(2);
    rleos.write(3);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 1, 2, 3 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testTwoDifferentBytesStream3() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0xfe);
    rleos.write(0xff);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 1, (byte)0xfe, (byte)0xff };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testTwoEqualsBytesStream1() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0);
    rleos.write(0);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -2, 0 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testTwoEqualsBytesStream2() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(1);
    rleos.write(1);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -2, 1 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testTwoEqualsBytesStream3() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0xff);
    rleos.write(0xff);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -2, (byte)0xff };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testEqualsDifferentBlockStream1() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0);
    rleos.write(0);
    rleos.write(1);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -2, 0, 0, 1 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testEqualsDifferentBlockStream2() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0);
    rleos.write(0);
    rleos.write(1);
    rleos.write(2);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -2, 0, 1, 1, 2 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testEqualsDifferentBlockStream3() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0);
    rleos.write(0);
    rleos.write(0xfe);
    rleos.write(0xff);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -2, 0, 1, (byte)0xfe, (byte)0xff };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testEqualsDifferentBlockStream4() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0xff);
    rleos.write(0xff);
    rleos.write(0xfe);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -2, (byte)0xff, 0, (byte)0xfe };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testEqualsDifferentBlockStream5() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0xff);
    rleos.write(0xff);
    rleos.write(0xfe);
    rleos.write(0xfd);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -2, (byte)0xff, 1, (byte)0xfe, (byte)0xfd };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testDifferentEqualsBlockStream1() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0);
    rleos.write(1);
    rleos.write(1);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 0, 0, -2, 1 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testDifferentEqualsBlockStream2() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0);
    rleos.write(1);
    rleos.write(2);
    rleos.write(2);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 1, 0, 1, -2, 2 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testDifferentEqualsBlockStream3() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0xfe);
    rleos.write(0xff);
    rleos.write(0);
    rleos.write(0);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 1, (byte)0xfe, (byte)0xff, -2, 0 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testDifferentEqualsBlockStream4() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0xfe);
    rleos.write(0xff);
    rleos.write(0xff);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 0, (byte)0xfe, -2, (byte)0xff };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testDifferentEqualsBlockStream5() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0xfe);
    rleos.write(0xfd);
    rleos.write(0xff);
    rleos.write(0xff);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 1, (byte)0xfe, (byte)0xfd, -2, (byte)0xff };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }



























  public void testEqualsDifferentEqualsBlockStream1() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0);
    rleos.write(0);
    rleos.write(1);
    rleos.write(0);
    rleos.write(0);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -2, 0, 0, 1, -2, 0 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testEqualsDifferentEqualsBlockStream2() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0);
    rleos.write(0);
    rleos.write(1);
    rleos.write(2);
    rleos.write(0);
    rleos.write(0);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -2, 0, 1, 1, 2, -2, 0 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testEqualsDifferentEqualsBlockStream3() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0);
    rleos.write(0);
    rleos.write(0xfe);
    rleos.write(0xff);
    rleos.write(0);
    rleos.write(0);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -2, 0, 1, (byte)0xfe, (byte)0xff, -2, 0 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testEqualsDifferentEqualsBlockStream4() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0xff);
    rleos.write(0xff);
    rleos.write(0xfe);
    rleos.write(0xff);
    rleos.write(0xff);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -2, (byte)0xff, 0, (byte)0xfe, -2, (byte)0xff };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testEqualsDifferentEqualsBlockStream5() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0xff);
    rleos.write(0xff);
    rleos.write(0xfe);
    rleos.write(0xfd);
    rleos.write(0xff);
    rleos.write(0xff);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -2, (byte)0xff, 1, (byte)0xfe, (byte)0xfd, -2, (byte)0xff };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testDifferentEqualsDifferentBlockStream1() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0);
    rleos.write(1);
    rleos.write(1);
    rleos.write(0);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 0, 0, -2, 1, 0, 0 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testDifferentEqualsDifferentBlockStream2() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0);
    rleos.write(1);
    rleos.write(2);
    rleos.write(2);
    rleos.write(0);
    rleos.write(1);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 1, 0, 1, -2, 2, 1, 0, 1 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testDifferentEqualsDifferentBlockStream3() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0xfe);
    rleos.write(0xff);
    rleos.write(0);
    rleos.write(0);
    rleos.write(0xfe);
    rleos.write(0xff);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 1, (byte)0xfe, (byte)0xff, -2, 0, 1, (byte)0xfe, (byte)0xff };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testDifferentEqualsDifferentBlockStream4() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0xfe);
    rleos.write(0xff);
    rleos.write(0xff);
    rleos.write(0xfe);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 0, (byte)0xfe, -2, (byte)0xff, 0, (byte)0xfe };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testDifferentEqualsDifferentBlockStream5() throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(0xfe);
    rleos.write(0xfd);
    rleos.write(0xff);
    rleos.write(0xff);
    rleos.write(0xfe);
    rleos.write(0xfd);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { 1, (byte)0xfe, (byte)0xfd, -2, (byte)0xff, 1, (byte)0xfe, (byte)0xfd };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testLargeEqualsStream1() throws IOException {
    byte[] data = new byte[128];
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -128, 0 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testLargeEqualsStream2() throws IOException {
    byte[] data = new byte[129];
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -128, 0, 0, 0 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  //ta wersja nie jest generowana
  /*public void testLargeEqualsStream3() throws IOException {
    byte[] data = new byte[129];
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -128, 0, -1, 0 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }*/

  public void testLargeEqualsStream4() throws IOException {
    byte[] data = new byte[130];
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -128, 0, -2, 0 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testLargeEqualsStream5() throws IOException {
    byte[] data = new byte[128];
    for (int i = 0; i < data.length; i++) {
      data[i] = (byte)0xff;
    }
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -128, (byte)0xff };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testLargeEqualsStream6() throws IOException {
    byte[] data = new byte[129];
    for (int i = 0; i < data.length; i++) {
      data[i] = (byte)0xff;
    }
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -128, (byte)0xff, 0, (byte)0xff };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  //ta wersja nie jest generowana
  /*public void testLargeEqualsStream7() throws IOException {
    byte[] data = new byte[129];
    for (int i = 0; i < data.length; i++) {
      data[i] = (byte)0xff;
    }
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -128, (byte)0xff, -1, (byte)0xff };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }*/

  public void testLargeEqualsStream8() throws IOException {
    byte[] data = new byte[130];
    for (int i = 0; i < data.length; i++) {
      data[i] = (byte)0xff;
    }
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -128, (byte)0xff, -2, (byte)0xff };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testLargeDifferentStream1() throws IOException {
    byte[] data = new byte[128];
    for (int i = 0; i < data.length; i++) {
      data[i] = (byte)i;
    }
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[129];
    exp[0] = 127;
    System.arraycopy(data, 0, exp, 1, 128);
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testLargeDifferentStream2() throws IOException {
    byte[] data = new byte[129];
    for (int i = 0; i < data.length; i++) {
      data[i] = (byte)i;
    }
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[131];
    exp[0] = 127;
    System.arraycopy(data, 0, exp, 1, 128);
    exp[129] = 0;
    exp[130] = (byte)128;
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  //ta wersja nie jest generowana
  /*public void testLargeDifferentStream3() throws IOException {
    byte[] data = new byte[129];
    for (int i = 0; i < data.length; i++) {
      data[i] = (byte)i;
    }
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[131];
    exp[0] = 127;
    System.arraycopy(data, 0, exp, 1, 128);
    exp[129] = -1;
    exp[130] = (byte)128;
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }*/

  public void testLargeDifferentStream4() throws IOException {
    byte[] data = new byte[130];
    for (int i = 0; i < data.length; i++) {
      data[i] = (byte)i;
    }
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[132];
    exp[0] = 127;
    System.arraycopy(data, 0, exp, 1, 128);
    exp[129] = 1;
    System.arraycopy(data, 128, exp, 130, 2);
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testLargeEqualsDifferentStream1() throws IOException {
    byte[] data = new byte[129];
    data[128] = 1;
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -128, 0, 0, 1 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testLargeEqualsDifferentStream2() throws IOException {
    byte[] data = new byte[129];
    for (int i = 0; i < 128; i++) {
      data[i] = (byte)0xff;
    }
    data[128] = (byte)0xfe;
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -128, (byte)0xff, 0, (byte)0xfe };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testLargeDifferentEqualsStream() throws IOException {
    byte[] data = new byte[130];
    for (int i = 0; i < 128; i++) {
      data[i] = (byte)i;
    }
    for (int i = 128; i < data.length; i++) {
      data[i] = (byte)128;
    }
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[131];
    exp[0] = 127;
    System.arraycopy(data, 0, exp, 1, 128);
    exp[129] = -2;
    exp[130] = (byte)128;
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }

  public void testLargeEqualsEqualsStream() throws IOException {
    byte[] data = new byte[130];
    for (int i = 128; i < data.length; i++) {
      data[i] = (byte)128;
    }
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    OutputStream rleos = new RLEOutputStream(baos);
    rleos.write(data);
    rleos.close();
    byte[] a = baos.toByteArray();
    assertNotNull(a);
    byte[] exp = new byte[] { -128, 0, -2, (byte)128 };
    assertEquals(exp.length, a.length);
    assertTrue(Arrays.equals(exp, a));
  }
}
